/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.helloworld.helloworld2;


import cz.zcu.fav.kiv.jsim.JSimException;
import cz.zcu.fav.kiv.jsim.JSimInvalidParametersException;
import cz.zcu.fav.kiv.jsim.JSimProcess;
import cz.zcu.fav.kiv.jsim.JSimSimulation;
import cz.zcu.fav.kiv.jsim.JSimSimulationAlreadyTerminatedException;
import cz.zcu.fav.kiv.jsim.JSimTooManyProcessesException;

/**
 *  A simple Hello-World example. One process is created and activated. Now, we did not forget!
 *
 */
public class HelloWorld2
{
	public static void main(String[] args)
	{
		JSimSimulation simulation = null;
		HelloWorld2Process process = null;

		try
		{
			// First, we create a simulation and one process.
			simulation = new JSimSimulation("Hello World 2 Simulation");
			process = new HelloWorld2Process("Hello World 2 Process", simulation);

			// We must not forget to activate the process. A nice time, isn't it?
			process.activate(1.2345);

			// Now we run the simulation. We will do two steps only (the second one won't do anything) although it is not obvious here.
			while (simulation.step() == true)
				;
		} // try

		// All possible exceptions are caught here in one handler.
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch

		// It is a good practice to put the shutdown in the finally block. Then you can be sure that that the JVM will not "hang". (This wouldn't occur here.)
		finally
		{
			simulation.shutdown();
		}
	} // main
	
} // class HelloWorld2

/**
 *  A simple process example.
 */
class HelloWorld2Process extends JSimProcess
{
	/**
	 *  Note that you must ALWAYS write your own constructor &ndash; JSimProcess' one has two parameters.
	 */
	public HelloWorld2Process(String name, JSimSimulation sim) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
	{
		super(name, sim);
	} // constructor

	/**
	 *  This is the life of the process. You can do whatever you want here.
	 */
	protected void life()
	{
		message("Hello! It is " + myParent.getCurrentTime() + " o'clock now.");
		message("Good - Bye!");
	} // life

} // class HelloWorld2Process
