/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.helloworld.helloworld3;


import cz.zcu.fav.kiv.jsim.JSimException;
import cz.zcu.fav.kiv.jsim.JSimInvalidParametersException;
import cz.zcu.fav.kiv.jsim.JSimProcess;
import cz.zcu.fav.kiv.jsim.JSimSimulation;
import cz.zcu.fav.kiv.jsim.JSimSimulationAlreadyTerminatedException;
import cz.zcu.fav.kiv.jsim.JSimTooManyProcessesException;

/**
 * A simple Hello-World example. One process is created and activated. It prints ten times current time and then let other processes run
 * while it is sleeping.
 * 
 */
public class HelloWorld3
{
	public static void main(String[] args)
	{
		JSimSimulation simulation = null;
		HelloWorld3Process process = null;

		try
		{
			// First, we create a simulation and one process.
			simulation = new JSimSimulation("Hello World 3 Simulation");
			process = new HelloWorld3Process("Hello World 3 Process", simulation);

			// We must not forget to activate the process. A nice time, isn't it?
			process.activate(1.2345);

			// Now we run the simulation. We will do as many steps as it is needed by all processes present in the system. This is
			// potentially dangerous if there is at least one process which never wants to finish.
			while (simulation.step() == true)
				;
		} // try

		// All three possible exceptions are caught here in one handler
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch

		// It is a good practice to put the shutdown in the finally block. Then you can be sure that that the JVM will not "hang". (This wouldn't occur here.)
		finally
		{
			simulation.shutdown();
		}
	} // main
} // HelloWorld3

/**
 *  A simple process example. It prints current simulation time several times and then dies.
 */
class HelloWorld3Process extends JSimProcess
{
	/**
	 *  Note that you must ALWAYS write your own constructor &ndash; JSimProcess' one has two parameters.
	 */
	public HelloWorld3Process(String name, JSimSimulation sim) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
	{
		super(name, sim);
	} // constructor

	/**
	 *  This is the life of the process. We will cycle here ten times.
	 */
	protected void life()
	{
		try
		{
			int i;

			message("Hello! I have just been born!");

			for (i = 1; i < 10; i++)
			{
				message("Hello! It is " + myParent.getCurrentTime() + " o'clock now and I'm going to sleep for a while.");
				// Now we let other processes run. That's a pity we are alone here!
				hold(9.8765);
			} // for i

			message("Good - Bye! My time is over.");
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch
	} // life

} // class HelloWorld3Process
