/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.mistakesanderrors.allowedcancel;


import cz.zcu.fav.kiv.jsim.JSimException;
import cz.zcu.fav.kiv.jsim.JSimInvalidParametersException;
import cz.zcu.fav.kiv.jsim.JSimProcess;
import cz.zcu.fav.kiv.jsim.JSimSimulation;
import cz.zcu.fav.kiv.jsim.JSimSimulationAlreadyTerminatedException;
import cz.zcu.fav.kiv.jsim.JSimTooManyProcessesException;

/**
 * A counter-example to ForbiddenHold and ForbiddenPassivate which demonstrates that it is possible (and useful!) to cancel another process'
 * scheduled events.
 * 
 */
public class AllowedCancel
{
	public static void main(String[] args)
	{
		JSimSimulation simulation = null;
		CancelProcess process1 = null, process2 = null;
		int i;

		try
		{
			// First, we create a simulation and two processes.
			simulation = new JSimSimulation("Allowed Cancel Simulation");
			process1 = new CancelProcess("Guy No 1", simulation, null);
			process2 = new CancelProcess("Guy No 2", simulation, process1);

			// We must not forget to activate both processes.
			process1.activate(1.0);
			process2.activate(2.0);

			// Now we do just seven steps. It is enough to show the feature.
			for (i = 0; (i < 7) && (simulation.step() == true); i++)
				;
		} // try
	
		// All possible exceptions are caught here in one handler
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment();
		} // catch
		
		// It is a good practice to put the shutdown in the finally block. Then you can be sure that the JVM will not hang.
		finally
		{
			simulation.shutdown();
		} // finally
	} // main

} // class AllowedCancel

/**
 * A simple process example. It tries to passivate another process but is not successful.
 */ 
class CancelProcess extends JSimProcess
{
	private CancelProcess procToCancel;

	/**
	 * Note that you must ALWAYS write your own constructor &ndash; JSimProcess' one has two parameters.
	 */ 
	public CancelProcess(String name, JSimSimulation sim, CancelProcess proc) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
	{
		super(name, sim);
		procToCancel = proc;
	} // constructor

	/**
	 * This is the life of the process. We will try to passivate the other guy or print hello.
	 */ 
	protected void life()
	{
		try
		{
			int i;

			// Are we good or bad?
			if (procToCancel != null) // Bad
			{
				message("It is " + myParent.getCurrentTime() + " o'clock.");
				message(getName() + ": Hello! I'm going to cancel " + procToCancel.getName() + "!");

				// If you comment out following line, everything will be OK and the second process will run as it wishes.
				procToCancel.cancel();
				message(getName() + ": It's done. It will not wake up anymore. Bye.");
			} // if
			else
			// Good
			{
				message(getName() + ": Hello! It's " + myParent.getCurrentTime() + " o'clock.");
				for (i = 0; i < 5; i++)
				{
					message(getName() + ": I'm still here. See you in 0.45 sec.");
					hold(0.45);
				} // for

				message("It is " + myParent.getCurrentTime() + " o'clock.");
				message(getName() + ": Good-bye.");
			} // else
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment();
		} // catch
	} // life

} // class CancelProcess
