/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.queues.producerconsumer;

import cz.zcu.fav.kiv.jsim.JSimException;
import cz.zcu.fav.kiv.jsim.JSimHead;
import cz.zcu.fav.kiv.jsim.JSimSimulation;

/**
 * A simplified producer-consumer example. Two processes share the same queue, one puts data into it and the other retrieves them.
 * 
 */
public class ProducerConsumerExample
{
	public static void main(String[] args)
	{
		JSimSimulation sim = null;
		JSimHead queue = null;
		ProducerProcess producer = null;
		ConsumerProcess consumer = null;
		int i;

		try
		{
			// It is necessary to create a simulation before creating a queue -- the queue needs simulation's time functions.
			sim = new JSimSimulation("Producer-Consumer Simulation");
			queue = new JSimHead("Shared Queue", sim);

			// Let's create the processes.
			producer = new ProducerProcess("Producer", sim, queue, 0.25);
			consumer = new ConsumerProcess("Consumer", sim, queue, 0.25);

			// Consumer will be started first, but he will switch to producer.
			producer.activate(1.0);
			consumer.activate(0.0);

			// Now, we will do 100 steps to show that it works.
			for (i = 0; (i < 100) && (sim.step() == true); i++)
				;

			sim.message("Bye!");
		} // try

		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		}

		// We must not forget to shut the simulation down!
		finally
		{
			sim.shutdown();
		}
	} // main

} // class ProducerConsumerExample
