/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.queues.statistics;

import cz.zcu.fav.kiv.jsim.JSimException;
import cz.zcu.fav.kiv.jsim.JSimHead;
import cz.zcu.fav.kiv.jsim.JSimInvalidParametersException;
import cz.zcu.fav.kiv.jsim.JSimLink;
import cz.zcu.fav.kiv.jsim.JSimProcess;
import cz.zcu.fav.kiv.jsim.JSimSimulation;
import cz.zcu.fav.kiv.jsim.JSimSimulationAlreadyTerminatedException;
import cz.zcu.fav.kiv.jsim.JSimTooManyProcessesException;

/**
 * A simple example on queue statistics. A process inserts several items to a queue and measures the average length and the average waiting
 * time of the queue.
 * 
 */
public class StatisticsExample
{
	public static void main(String[] args)
	{
		JSimSimulation sim = null;
		JSimHead queue;
		ProcessWithAQueue process;

		try
		{
			// It is necessary to create a simulation before creating a queue and a process.
			sim = new JSimSimulation("Simulation with queues");

			queue = new JSimHead("Little queue", sim);
			process = new ProcessWithAQueue("Statistics Test Process", sim, queue);
			process.activate(0.0);

			// Now, let's do as many steps as possible.
			while (sim.step() == true)
				;
		} // try
		
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch

		// We must not forget... you know what.
		finally
		{
			sim.shutdown();
		} // finally
	} // main
	
} // class StatisticsExample

class ProcessWithAQueue extends JSimProcess
{
	private JSimHead queue;

	/**
	 *  Note that you must ALWAYS write your own constructor &ndash; JSimProcess' one has two parameters.
	 */
	public ProcessWithAQueue(String name, JSimSimulation sim, JSimHead aQueue) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
	{
		super(name, sim);
		queue = aQueue;
	} // constructor

	/**
	 * Displays information about the queue.
	 */
	protected void printInfo()
	{
		message("Time = " + myParent.getCurrentTime());
		message("Number of items in the queue = " + queue.cardinal());
		message("Lw     = " + queue.getLw());
		message("Tw     = " + queue.getTw());
		message("Tw all = " + queue.getTwForAllLinks());
		message("------------------------------------------------------");
	} // printInfo

	/**
	 * This is the life of the process.
	 */
	protected void life()
	{
		try
		{
			JSimLink[] links;
			int i;

			links = new JSimLink[10];
			for (i = 0; i < links.length; i++)
				links[i] = new JSimLink(new Integer(i + 1));

			message("Hello!");
			printInfo();
			links[0].into(queue);
			printInfo();
			hold(1.0); // Jump to 1

			printInfo();
			links[1].into(queue);
			links[2].into(queue);
			printInfo();
			hold(1.0); // Jump to 2

			printInfo();
			// Nothing
			printInfo();
			hold(3.0); // Jump to 5

			printInfo();
			links[0].out();
			links[3].into(queue);
			printInfo();
			hold(2.0); // Jump to 7

			printInfo();
			links[4].into(queue);
			links[5].into(queue);
			links[6].into(queue);
			printInfo();
			hold(3.0); // Jump to 10

			printInfo();
			links[5].out();
			links[6].out();
			printInfo();
			hold(140.0); // Jump to 150

			printInfo();
			links[7].into(queue);
			links[8].into(queue);
			links[9].into(queue);
			printInfo();
			hold(15.0); // Jump to 165

			printInfo();
			links[7].out();
			links[8].out();
			links[9].out();
			printInfo();
			hold(1.0); // Jump to 166

			printInfo();
			links[2].out();
			links[3].out();
			links[4].out();
			printInfo();
			hold(955.0); // Jump to 1121

			printInfo();
			links[1].out();
			printInfo();
			message("Good - Bye!");
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch
	} // life

} // class ProcessWithAQueue
