/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.queueingnetworks;

import cz.zcu.fav.kiv.jsim.JSimException;
import cz.zcu.fav.kiv.jsim.JSimInvalidParametersException;
import cz.zcu.fav.kiv.jsim.JSimLink;
import cz.zcu.fav.kiv.jsim.JSimProcess;
import cz.zcu.fav.kiv.jsim.JSimSimulation;
import cz.zcu.fav.kiv.jsim.JSimSimulationAlreadyTerminatedException;
import cz.zcu.fav.kiv.jsim.JSimSystem;
import cz.zcu.fav.kiv.jsim.JSimTooManyProcessesException;

/**
 * A simple generator. Creates new transactions and puts them into a queue. Look into the documentation for additional information.
 * 
 */
public class Generator extends JSimProcess
{
	private double lambda;
	private QueueWithServer queue;

	public Generator(String name, JSimSimulation sim, double l, QueueWithServer q) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
	{
		super(name, sim);
		lambda = l;
		queue = q;
	} // constructor

	protected void life()
	{
		JSimLink link;

		try
		{
			while (true)
			{
				// Periodically creating new transactions and putting them into the queue.
				link = new JSimLink(new Transaction(myParent.getCurrentTime()));
				link.into(queue);
				if (queue.getServer().isIdle())
					queue.getServer().activate(myParent.getCurrentTime());
				hold(JSimSystem.negExp(lambda));
			} // while
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch
	} // life

} // class Generator
