/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *  Copyright (c) 2003 Pavel Domecký
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.gui.interactivemode.finitesimulation;

import cz.zcu.fav.kiv.jsim.JSimException;
import cz.zcu.fav.kiv.jsim.JSimInvalidParametersException;
import cz.zcu.fav.kiv.jsim.JSimProcess;
import cz.zcu.fav.kiv.jsim.JSimSimulation;
import cz.zcu.fav.kiv.jsim.JSimSimulationAlreadyTerminatedException;
import cz.zcu.fav.kiv.jsim.JSimSimulationMode;
import cz.zcu.fav.kiv.jsim.JSimSystem;
import cz.zcu.fav.kiv.jsim.JSimTooManyProcessesException;

/**
 * A simple example using GUI output.
 * 
 * The interactive GUI mode is used here, specified as the 2nd parameter of the simulation's constructor. Five processes are created and
 * activated, but no step is done. Instead we switch to graphic mode where the user can interact with the simulation using the mouse.
 */
public class GUIGreetings
{
	public static void main(String[] args)
	{
		JSimSimulation simulation = null;
		GUIGreetingsProcess processA = null, processB = null, processC = null, processD = null, processE = null;

		try
		{
			// As usual, we must create a simulation first.
			System.out.println("Starting up simulation...");
			simulation = new JSimSimulation("Hello GUI World Simulation", JSimSimulationMode.GUI_INTERACTIVE);

			// Now creating and activating processes in exactly the same way as we do it using console output.
			simulation.message("Creating five processes...");
			processA = new GUIGreetingsProcess("Hello!", simulation, 0.5);
			processB = new GUIGreetingsProcess("Ciao!", simulation, 0.6);
			processC = new GUIGreetingsProcess("Salut!", simulation, 1.5);
			processD = new GUIGreetingsProcess("Hej!", simulation, 1.0);
			processE = new GUIGreetingsProcess("Ahoj!", simulation, 1.2);

			processA.activate(0.0);
			processB.activate(0.5);
			processC.activate(1.0);
			processD.activate(1.2);
			processE.activate(1.5);

			// The difference is here:
			// No calls of step(), just one call of runGUI(). That's it!
			// This simulation will terminate after some calls of step() from GUI.
			simulation.runGUI();
			simulation.message("Back on the console, good-bye.");
		} // try
		
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment();
		} // catch

		// As usual, don't forget to shutdown the simulation. Do it always in the finally block.
		finally
		{
			simulation.shutdown();
		} // finally
	} // main
} // class GUIGreetings

/**
 * An example of simple process. Writes "Hello" or its equivalent and then sleeps. Sometimes it decides to die. This process could be use in
 * a console-output simulation too, there is no difference.
 */
class GUIGreetingsProcess extends JSimProcess
{
	private String helloWord;
	private double lambda;

	public GUIGreetingsProcess(String hello, JSimSimulation sim, double l) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
	{
		super("Process that says " + hello, sim);
		helloWord = hello;
		lambda = l;
	} // constructor

	/**
	 * Note that we don't care at all about the mode we run in. We don't even know about it!
	 */ 
	protected void life()
	{
		double timeToSleep;

		try
		{
			// To die or to live, that's the question!
			while (JSimSystem.draw(0.95))
			{
				timeToSleep = JSimSystem.negExp(lambda);
				message(helloWord + " I'm going to sleep for " + timeToSleep + " seconds.");
				hold(timeToSleep);
			} // while

			error(helloWord + " I'm dying! Good-bye.");
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment();
		} // catch
	} // life

} // class GUIGreetingsProcess
