/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.gui.changeoutputinfo.custominfo;

import cz.zcu.fav.kiv.jsim.JSimException;
import cz.zcu.fav.kiv.jsim.JSimInvalidParametersException;
import cz.zcu.fav.kiv.jsim.JSimLink;
import cz.zcu.fav.kiv.jsim.JSimProcess;
import cz.zcu.fav.kiv.jsim.JSimSimulation;
import cz.zcu.fav.kiv.jsim.JSimSimulationAlreadyTerminatedException;
import cz.zcu.fav.kiv.jsim.JSimSystem;
import cz.zcu.fav.kiv.jsim.JSimTooManyProcessesException;

/**
 * Periodically takes transactions from its queue and inserts them in another queue or deletes them.
 * 
 */
public class Server extends JSimProcess
{
	private double mi;
	private double p;
	private QueueWithServer queueIn;
	private QueueWithServer queueOut;

	private int counter;
	private double transTq;

	public Server(String name, JSimSimulation sim, double parMi, double parP, QueueWithServer parQueueIn, QueueWithServer parQueueOut) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
	{
		super(name, sim);
		mi = parMi;
		p = parP;
		queueIn = parQueueIn;
		queueOut = parQueueOut;

		counter = 0;
		transTq = 0.0;
	} // constructor

	protected void life()
	{
		Transaction t;
		JSimLink link;

		try
		{
			while (true)
			{
				if (queueIn.empty())
				{
					// If we have nothing to do, we sleep.
					passivate();
				}
				else
				{
					// Simulating hard work here...
					hold(JSimSystem.negExp(mi));
					link = queueIn.first();

					// Now we must decide whether to throw the transaction away or to insert it into another queue.
					if (JSimSystem.uniform(0.0, 1.0) > p)
					{
						t = (Transaction) link.getData();
						counter++;
						transTq += myParent.getCurrentTime() - t.getCreationTime();
						link.out();
						link = null;
					}
					else
					{
						link.out();
						link.into(queueOut);
						if (queueOut.getServer().isIdle())
							queueOut.getServer().activate(myParent.getCurrentTime());
					} // else throw away / insert
				} // else queue is empty / not empty
			} // while
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		}
	} // life

	public int getCounter()
	{
		return counter;
	} // getCounter

	public double getTransTq()
	{
		return transTq;
	} // getTransTq

} // class Server
