/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *  Copyright (c) 2003 Pavel Domecký
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.gui.changeoutputinfo.inheritedinfowindow;

import java.util.Collection;

import javax.swing.JDialog;

import cz.zcu.fav.kiv.jsim.JSimHead;
import cz.zcu.fav.kiv.jsim.JSimInvalidParametersException;
import cz.zcu.fav.kiv.jsim.JSimProcess;
import cz.zcu.fav.kiv.jsim.JSimSimulation;
import cz.zcu.fav.kiv.jsim.JSimTooManyHeadsException;
import cz.zcu.fav.kiv.jsim.gui.JSimMainWindow;
import cz.zcu.fav.kiv.jsim.gui.JSimPair;

/**
 * A queue having its own server which takes transactions out from the queue. This class overrides the createDetailedInfoWindow() method in
 * order to replace the default window with its own one.
 */
public class QueueWithServer extends JSimHead
{
	private JSimProcess server;

	public QueueWithServer(String name, JSimSimulation sim, JSimProcess s) throws JSimInvalidParametersException, JSimTooManyHeadsException
	{
		super(name, sim);
		server = s;
	} // constructor

	public JSimProcess getServer()
	{
		return server;
	} // getServer

	public void setServer(JSimProcess s)
	{
		server = s;
	} // setServer

	/**
	 * Overrides the method from class JSimHead.
	 */
	public String getObjectListItemDescription()
	{
		return ("No.: " + getHeadNumber() + "  Name: " + getHeadName() + "  Elements: " + cardinal() + "  Average length: " + getLw());
	}

	/**
	 * Overriding the method from class JSimHead.
	 */
	public Collection<JSimPair> getDetailedInformationArray()
	{
		Collection<JSimPair> c;
		c = super.getDetailedInformationArray();
		c.add(new JSimPair("Average waiting time for all links:", Double.toString(getTwForAllLinks())));
		return c;
	}

	/**
	 * Overrides the method from class JSimHead. Creates a window that shows completely different information from the default one. Always
	 * create your detailed info window this way!
	 */
	public JDialog createDetailedInfoWindow(JSimMainWindow parentWindow)
	{
		MyGUIDetailedInfoWindow dWindow = new MyGUIDetailedInfoWindow(parentWindow, this);
		return dWindow;
	}

} // class QueueWithServer
