/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.ipc.messagepassing.asynchronousindirect3;

import cz.zcu.fav.kiv.jsim.JSimException;
import cz.zcu.fav.kiv.jsim.JSimSimulation;
import cz.zcu.fav.kiv.jsim.ipc.JSimMessageBox;

/**
 * A simple simulation of two processes exchanging messages asynchronously and indirectly via two message boxes. The messages know neither
 * the sender nor the receiver. Neither the sending nor the receiving operation blocks. If no message is found in the message box by the
 * receiving process, the process just goes on. The sender randomly chooses a message box to send the message to. So does the receiver.
 */
public class AsynchronousIndirectCommunication
{
	public static void main(String[] args)
	{
		JSimSimulation simulation = null;
		JSimMessageBox box1 = null;
		JSimMessageBox box2 = null;
		SendingProcess sender;
		ReceivingProcess receiver;

		try
		{
			System.out.println("Initializing the simulation...");
			simulation = new JSimSimulation("Asynchronous Indirect Communication Simulation");
			box1 = new JSimMessageBox("Shared Message Box 1");
			box2 = new JSimMessageBox("Shared Message Box 2");
			sender = new SendingProcess("Sender", simulation, box1, box2);
			receiver = new ReceivingProcess("Receiver", simulation, box1, box2);

			simulation.message("Activating the processes...");
			sender.activate(0.0);
			receiver.activate(0.0);

			simulation.message("Running the simulation, please wait.");

			while ((simulation.getCurrentTime() < 100.0) && (simulation.step() == true))
				;
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch
		finally
		{
			simulation.shutdown();
		} // finally
	} // main

} // class AsynchronousIndirectCommunication
