/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.ipc.messagepassing.synchronoussymmetric;

import cz.zcu.fav.kiv.jsim.JSimException;
import cz.zcu.fav.kiv.jsim.JSimInvalidParametersException;
import cz.zcu.fav.kiv.jsim.JSimProcess;
import cz.zcu.fav.kiv.jsim.JSimSimulation;
import cz.zcu.fav.kiv.jsim.JSimSimulationAlreadyTerminatedException;
import cz.zcu.fav.kiv.jsim.JSimSystem;
import cz.zcu.fav.kiv.jsim.JSimTooManyProcessesException;
import cz.zcu.fav.kiv.jsim.ipc.JSimSymmetricMessage;

/**
 * A simple process periodically sending messages to another process. The send operation is blocking. If the receiver is not ready to
 * receive a message, the sender blocks. After every insertion, the sending process sleeps for a while.
 */
public class SendingProcess extends JSimProcess
{
	public static final double LAMBDA = 1.0;

	private JSimProcess receiver;

	// ------------------------------------------------------------------------------------------------------------------------------------

	public SendingProcess(String name, JSimSimulation parent, JSimProcess receivingProcess) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
	{
		super(name, parent);
		receiver = receivingProcess;
	} // constructor

	protected void life()
	{
		JSimSymmetricMessage message;
		int counter = 0;
		double sleepTime;

		try
		{
			while (true)
			{
				message = new JSimSymmetricMessage(this, receiver, new Integer(counter));
				message(myParent.getCurrentTime() + " - " + getName() + ": Sending a message with data `" + counter + "' to process `" + receiver.getName() + "'.");
				sendMessageWithBlocking(message);
				message(myParent.getCurrentTime() + " - " + getName() + ": Mesage sent, sender is running again.");
				counter++;
				sleepTime = JSimSystem.negExp(LAMBDA);
				message(myParent.getCurrentTime() + " - " + getName() + ": Going to sleep for " + sleepTime + ".");
				hold(sleepTime);
			} // while
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch
	} // life

} // class SendingProcess
