/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.ipc.messagepassing.synchronoussymmetric;

import cz.zcu.fav.kiv.jsim.JSimException;
import cz.zcu.fav.kiv.jsim.JSimSimulation;

/**
 * A simple simulation of two processes exchanging messages synchronously and directly (process to process). The messages know both the
 * sender and the receiver. Both the sending and the receiving operations block. If no message is found in the default message box of the
 * receiving process, the process blocks until the sender sends it a message. The sender blocks during send if the receiver is not ready to
 * receive a message. The sender remains blocked until the receiver performs a receive operation.
 */
public class SynchronousSymmetricCommunication
{
	public static void main(String[] args)
	{
		JSimSimulation simulation = null;
		SendingProcess sender;
		ReceivingProcess receiver;

		try
		{
			System.out.println("Initializing the simulation...");
			simulation = new JSimSimulation("Synchronous Symmetric Communication Simulation");
			receiver = new ReceivingProcess("Receiver", simulation);
			sender = new SendingProcess("Sender", simulation, receiver);

			simulation.message("Activating the processes...");
			sender.activate(0.0);
			receiver.activate(0.0);

			simulation.message("Running the simulation, please wait.");

			while ((simulation.getCurrentTime() < 100.0) && (simulation.step() == true))
				;
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch
		finally
		{
			simulation.shutdown();
		} // finally
	} // main

} // class SynchronousSymmetricCommunication
