/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.ipc.messagepassing.synchronouswithknownreceiver;

import cz.zcu.fav.kiv.jsim.JSimException;
import cz.zcu.fav.kiv.jsim.JSimSimulation;

/**
 * A simple simulation of three processes (2 senders, 1 receiver) exchanging messages synchronously and directly. The messages know only the
 * receiver, not the sender. Both the sending and the receiving operations block. If no message is found in its default message box by the
 * receiving process, the process blocks until a sender sends it a message. The sender blocks during send if the receiver is not ready to
 * receive a message from this specific process. The sender remains blocked until the receiver performs a receive operation.
 */
public class SynchronousCommunicationWithKnownReceiver
{
	public static void main(String[] args)
	{
		JSimSimulation simulation = null;
		SendingProcess sender1;
		SendingProcess sender2;
		ReceivingProcess receiver;

		try
		{
			System.out.println("Initializing the simulation...");
			simulation = new JSimSimulation("Synchronous Communication with Known Receiver Simulation");
			receiver = new ReceivingProcess("Receiver", simulation);
			sender1 = new SendingProcess("Sender 1", simulation, receiver, 'A');
			sender2 = new SendingProcess("Sender 2", simulation, receiver, 'B');

			simulation.message("Activating the processes...");
			sender1.activate(0.0);
			sender2.activate(0.0);
			receiver.activate(0.0);

			simulation.message("Running the simulation, please wait.");

			while ((simulation.getCurrentTime() < 100.0) && (simulation.step() == true))
				;
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch
		finally
		{
			simulation.shutdown();
		} // finally
	} // main

} // class SynchronousCommunicationWithKnownReceiver
